/*
 * FILE: astronave.c
 * AUTORE: Alessio Pani
 * DATA: 27/06/2009
 * PROGETTO: Space Invaders - Versione 1
 * DESCRIZIONE: Definizione della funzione di gestione del movimento dell'astronave
 */

#ifndef strutt_comunicaz_c_incluso
  #include "strutt_comunicaz.c"
#endif

#include<stdio.h>
#include<sys/wait.h>


/**
 * Procedura che gestisce il movimento di un'astronave.
 * Legge gli input da tastiera e comunica la propria posizione al controllo attraverso la pipe
 * Consente di sparare dei missili che comunicheranno autonomamente le proprie coordinate al controllo.
 * Parametri:
 *   pipeout: descrittore di scrittura della pipe per poter trasmettere al controllo i movimenti effettuati.
 *   righe_schermo: Righe dello schermo
 *   colonne_schermo: Colonne dello schermo
 *   tasto_destra: Codice del tasto da utlizzare per spostare a destra l'astronave
 *   tasto_sinistra: Codice del tasto da utlizzare per spostare a sinistra l'astronave
 *   tasto_spara: Codice del tasto da utlizzare per sparare
 *   max_missili: Numero massimo di missili lanciabili prima della scomparsa di quelli già lanciati dallo schermo
 *   passo: indica di quanto l'astronave deve spostarsi alla pressione di tasto_destra o tasto_sinistra
 *   tipo_astronave: indica il codice numerico da associare all'oggetto di tipo astronave.
 *   tipo_missile: indica il codice numerico da associare all'oggetto di tipo missile.
 * I codici di tipo_astronave e tipo_missile devono essere gli stessi passati al controllo
 */
void astronave(int pipeout, int righe_schermo, int colonne_schermo, int tasto_destra, int tasto_sinistra, int tasto_spara, int passo, int max_missili, int tipo_astronave, int tipo_missile){
  char c;
  int n_missile = 0;
  int pid_missili[max_missili];
  posizione pos; //Struttura per salvare la posizione corrente dell'astronave e mandarla al controllo
  pos.tipo = tipo_astronave;
  pos.x = (colonne_schermo / 2) - 3; //L'astronave viene creata in posizione centrale
  pos.y = righe_schermo - 6;         //e in fondo all'area disponibile
  
  int max_missili_raggiunto = 0; //Flag che indica che il numero massimo da associare ai missili è stato raggiunto almeno una volta

  write(pipeout, &pos, sizeof(pos)); //Invio della posizione iniziale al controllo

  while (1){
    c = getch();
    if((int)c == tasto_destra){
	pos.x += passo;
	if (pos.x > colonne_schermo - 7)
	  pos.x = colonne_schermo - 7;
    }

    if((int)c == tasto_sinistra){
      pos.x += -passo;
      if (pos.x < 0)
	pos.x = 0;
    }
   
    if((int)c == tasto_spara){
      int tempo = 50000;
      /* Si deve evitare che vengano lanciati insieme più missili di quelli consentiti. Per fare ciò, una volta che i numeri disponibili
       * sono stati utilizzati tutti, si verifica che il processo associato al missile col numero che vogliamo riutilizzare sia terminato.
       */
      if (!max_missili_raggiunto || waitpid(pid_missili[n_missile], NULL, WNOHANG) ){
        switch (pid_missili[n_missile] = fork()){
	  case -1:
	    break;
	  case 0:	  
	    switch (pid_missili[n_missile + 1] = fork()){
	      case 0:	        
	        missile(pipeout, pos.y, pos.x + 3, colonne_schermo, tempo, 1, n_missile + 1, tipo_missile);
	        exit(0);
	      default:
	        missile(pipeout, pos.y, pos.x + 3, colonne_schermo, tempo, -1, n_missile, tipo_missile);		
		waitpid(pid_missili[n_missile + 1], NULL, 0);
	        exit(0);
	    }
        }
        n_missile = (n_missile + 2)%max_missili;
	if (!max_missili_raggiunto && n_missile == 0)
          max_missili_raggiunto = 1;
      }
    } //Fine tasto spara
    if ((int)c == tasto_sinistra || (int)c == tasto_destra) //Evita l'invio di dati se non ci sono cambiamenti
      write(pipeout, &pos, sizeof(pos));
  }
}
