/*
 * FILE: bomba.c
 * AUTORE: Alessio Pani
 * DATA: 27/06/2009
 * PROGETTO: Space Invaders - Versione 1
 * DESCRIZIONE: Definizione della funzione di gestione del movimento di una bomba lanciata da un nemico
 */

#ifndef strutt_comunicaz_c_incluso
  #include "strutt_comunicaz.c"
#endif

/**
 * Procedura che gestisce il movimento di una bomba.
 * Scende in linea retta sullo schermo a partire dalla posizione iniziale
 * Parametri:
 *   pipeout: descrittore di scrittura della pipe per poter trasmettere al controllo i movimenti effettuati.
 *   riga: riga iniziale dalla quale iniziare la discesa
 *   colonna: colonna in cui si trova la bomba
 *   righe_schermo: Righe dello schermo
 *   tempo: indica l'intervallo di tempo in microsecondi prima dei quali fare ogni movimento verso il basso
 *   numero: numero della bomba. Serve perché il controllo possa distinguerla dalle altre
 *   tipo: indica il codice numerico da associare all'oggetto di tipo bomba.
 */
void bomba(int pipeout, int riga, int colonna, int righe_schermo, int tempo, int numero, int tipo){
  posizione pos; //Struttura per salvare la posizione corrente della bomba e mandarla al controllo
  pos.tipo = tipo;
  pos.x = colonna;
  pos.y = riga;
  pos.numero = numero;
  write(pipeout, &pos, sizeof(pos)); //Invio della posizione iniziale al controllo
  while (pos.y <= righe_schermo){
    pos.y++;
    write(pipeout, &pos, sizeof(pos)); //Invio della posizione al controllo
    usleep(tempo);
  }
} 
