/*
 * FILE: controllo.c
 * AUTORE: Alessio Pani
 * DATA: 27/06/2009
 * PROGETTO: Space Invaders - Versione 1
 * DESCRIZIONE: Definizione della funzione di controllo del gioco.
 * 	Tale funzione riceve le coordinate degli oggetti da mostrare sullo schermo a ogni loro spostamento
 *	e ne gestisce il disegno e le interazioni.
 */

#ifndef strutt_comunicaz_c_incluso
  #include "strutt_comunicaz.c"
#endif

#include<stdio.h>
#include<string.h>
/**
 * Funzione di controllo.
 * Riceve attraverso la pipe in ingresso le coordinate degli oggetti da mostrare, li riconosce tramite
 * il tipo associato, disegna gli oggetti sullo schermo e gestisce le interazioni fra essi.
 * Parametri:
 *  pipein: pipe dalla quale riceve dagli oggetti le proprie coordinate
 *  pipe_nemici: array di pipe per la comunicazione coi nemici. Al nemico n-simo corrisponde il descrittore n-simo
 *  righe_schermo: righe dello schermo
 *  colonne_schermo: colonne dello schermo
 *  max_nemici: numero di massimo di nemici da controllare
 *  max_missili: numero massimo di missili che possono esistere simultaneamente
 *  tipo_astronave: indica il codice numerico da associare all'oggetto di tipo astronave.
 *  tipo_missile: indica il codice numerico da associare all'oggetto di tipo missile.
 *  tipo_nemico: indica il codice numerico da associare all'oggetto di tipo nemico.
 *  tipo_bomba: indica il codice numerico da associare all'oggetto di tipo bomba.
 */
void controllo(int pipein, int* pipe_nemici, int righe_schermo, int colonne_schermo, int max_nemici, int max_missili, int tipo_astronave, int tipo_missile, int tipo_nemico, int tipo_bomba){

  /*Strutture in cui salvare le posizioni degli oggetti da mostrare*/
  posizione valore_letto, astronave;
  posizione nemici[max_nemici];
  posizione bombe[max_nemici];
  posizione missili[max_missili];

  /* Array di flag per lo stato dei missili. Il flag corrispondente a un missile vale 1 se ha colpito un nemico */
  int missile_colpito[max_missili];

  cont_nem com_nem; //Struttura per inviare messaggi ai nemici

  char sch_astronave[6][6] = { //Disegno dell'astronave. Dimensione max: 6x6
			    {' ', ' ', ' ', ' ', ' ', ' '},
			    {' ', ' ', ' ', ' ', ' ', ' '},
			    {' ', ' ', ' ', ' ', ' ', '#'},
			    {' ', ' ', ' ', ' ', '#', '#'},
			    {' ', ' ', ' ', '#', '#', '#'},
			    {' ', ' ', '#', '#', '#', '#'}
			   };
  int off_a_astronave = 2;  //Prima riga non vuota per il disegno dell'astronave
  int off_l_astronave = 2;  //Prima colonna non vuota per il disegno dell'astronave
  int a_astronave = 4;  //Altezza dell'astronave in caratteri
  int l_astronave = 4;  //Lunghezza dell'astronave in caratteri

  
  int livello_nemici[max_nemici]; //Fase di vita dei nemici 0: Morto,  1: Primo livello, 2: Secondo livello
  int nemici_colpiti[max_nemici]; //Array di flag che indicano che il nemico n-simo è appena stato colpito da un missile
  int nemici2l_colpiti[max_nemici * 4]; //Numero di colpi ricevuti da ciascuna navicella di secondo livello
  int len_nem_1 = 3; //Caratteri occupati dal nemico nella sua prima fase
  int len_nem_2 = 3; //Caratteri occupati dal nemico nella sua seconda fase
  int n_nem_2 = 4; //Numeri di navicelle nel gruppo di nemici nella seconda fase
  int nemici_distrutti = 0; //Tiene il conto dei nemici distrutti. Quando è uguale a max_nemici il giocatore ha vinto

  int game_over = 0; //0: Gioco in corso, 1: Perso, 2: Vinto

  int i, j, k; //Contatori vari;

  //Inizializzazione delle strutture per memorizzare lo stato del gioco
  for(i = 0; i < max_nemici; i++){
    nemici[i].x = -1;
    nemici[i].y = -1;
    bombe[i].x = -1;
    bombe[i].y = -1;
    livello_nemici[i] = 1;
    nemici_colpiti[i] = 0;    
    nemici2l_colpiti[n_nem_2 * i] = 0;    
    nemici2l_colpiti[n_nem_2 * i + 1] = 0;    
    nemici2l_colpiti[n_nem_2 * i + 2] = 0;    
    nemici2l_colpiti[n_nem_2 * i + 3] = 0;    
  }
  for(i = 0; i < max_missili; i++){
    missili[i].x = -1;
    missili[i].y = -1;
    missile_colpito[i] = 0;
  }
  astronave.x = -1;
  astronave.y = -1;

  do{
    read(pipein, &valore_letto, sizeof(valore_letto));

    /* CONTROLLO ASTRONAVE */
    if (valore_letto.tipo == tipo_astronave){
      /* Cancellazione dell'astronave nella posizione precedente */
      if (astronave.x >= 0)
        for (i = off_a_astronave; i < (off_a_astronave + a_astronave); i++)
          for (j = off_l_astronave; j < (off_l_astronave + l_astronave); j++)
	    if(sch_astronave[i][j] != ' ')
	      mvaddch(astronave.y + i, astronave.x + j, ' ');    
      astronave = valore_letto;      
      /* Disegno dell'astronave e controllo di eventuali collisioni con bombe lanciate dai nemici*/
      for (i = off_a_astronave; i < (off_a_astronave + a_astronave); i++)
        for (j = off_l_astronave; j < (off_l_astronave + l_astronave); j++)
	  if(sch_astronave[i][j] != ' '){
	    mvaddch(valore_letto.y + i, valore_letto.x + j, sch_astronave[i][j]);	    
	    for (k = 0; !game_over && k < max_nemici; k++)
	      if (bombe[k].y != -1 && valore_letto.y + i == bombe[k].y && valore_letto.x + j == bombe[k].x)
		game_over = 1;
	  }
    }
    /* FINE CONTROLLO ASTRONAVE */

    /* CONTROLLO MISSILI */
    if (valore_letto.tipo == tipo_missile){
      if(missili[valore_letto.numero].x >= 0)
	/* Cancellazione del missile nella posizione precedente */
        mvaddch(missili[valore_letto.numero].y, missili[valore_letto.numero].x, ' ');      
      
      missili[valore_letto.numero] = valore_letto;
     /* Controlla se ha colpito un nemico
      * Se il missile aveva già colpito un nemico, il processo associato è ancora attivo, per cui sta ancora generando
      * le coordinate, ma affinché il comportamento sia realistico ci si aspetta che non oltrepassi il nemico
      * che ha colpito e quindi viene ignorato.
      */
      for (i = 0; !missile_colpito[valore_letto.numero] && i < max_nemici; i++){	  
	if (livello_nemici[i] == 1 && valore_letto.y == nemici[i].y){
	  /* Il missile occupa un carattere, ma il nemico no, quindi si confrontano le coordinate del missile con tutta la lunghezza del nemico */
	  for(k = 0; !missile_colpito[valore_letto.numero] && k < len_nem_1; k++)
	    if (valore_letto.x == nemici[i].x + k){
	      nemici_colpiti[i] = 1;
	      missile_colpito[valore_letto.numero] = 1;
	    }
	  /* Se un nemico è stato colpito dal missile, lo cancella */
	  if (missile_colpito[valore_letto.numero])
            for(k = 0; k < len_nem_1; k++)
	      mvaddch(nemici[i].y, nemici[i].x + k, ' ');
	}
	/* Se il nemico è di secondo livello controlla subito che siano sulla stessa riga */
	if (livello_nemici[i] == 2 && valore_letto.y == nemici[i].y){
	  /* Per il nemico di secondo livello, bisogna controllare se il missile colpisce una delle navicelle ancora "vive" */
	  int navicella = 0; /* Numero di navicella considerata */
	  /* j rappresenta l'offset per il posizionamento all'inizio di ciascuna navicella nemica */	  
	  for (j = 0; !missile_colpito[valore_letto.numero] && j < n_nem_2 * len_nem_2 + n_nem_2 - 1; j += len_nem_2 + 1){
	    /* Prima controlla che la navicella non sia già stata distrutta */
	    if (nemici2l_colpiti[n_nem_2 * i + navicella] < 2){
	      /* k rappresenta l'offset per controllare se il missile si trova all'interno della navicella */
	      for (k = 0; !missile_colpito[valore_letto.numero] && k < len_nem_2; k++)
	        if (valore_letto.x == nemici[i].x + j + k){
		  missile_colpito[valore_letto.numero] = 1;
		  nemici2l_colpiti[n_nem_2 * i + navicella]++;
		  nemici_colpiti[i] = 1;
	        }
	      /* Se una navicella è stata colpita dal missile, la cancella */
	      if (missile_colpito[valore_letto.numero])
	        for(k = 0; k < len_nem_2; k++)
	          mvaddch(nemici[i].y, nemici[i].x + j + k, ' ');
	    }
	    navicella++;
	  }	  
	}
      }
      /* Disegno del missile */
      if (!missile_colpito[valore_letto.numero]) //Non lo disegna se ha già colpito un nemico
        mvaddch(valore_letto.y, valore_letto.x, '|');
      else
	if (valore_letto.y <= 0 || valore_letto.x >= colonne_schermo || valore_letto.x <= 0)
	  /* A questo punto, il missile che ha colpito un nemico ha finito la sua corsa ideale verso la fine dello schermo.
	   * Il suo stato viene ripristinato affinché il prissimo missile con lo stesso numero funzioni.*/
          missile_colpito[valore_letto.numero] = 0;      
    }
    /* FINE CONTROLLO MISSILI */

    /* CONTROLLO NEMICI */
    if (valore_letto.tipo == tipo_nemico){
      com_nem.c_traiettoria = 0;
      com_nem.c_livello = 0;
      com_nem.nav_distrutta = -1;
      com_nem.passo_su = 0;

      /* Non deve disegnarlo se è morto */
      int disegna = (livello_nemici[valore_letto.numero] != 0);

      /* Controlla se il nemico è stato colpito da un missile. Caso del nemico di primo livello */
      if ((livello_nemici[valore_letto.numero] == 1 && nemici_colpiti[valore_letto.numero])){
        com_nem.c_livello = 1;
	livello_nemici[valore_letto.numero] = 2; /* Il nemico entra nella seconda fase */
	disegna = 0;
	nemici_colpiti[valore_letto.numero] = 0; /* Reimposta il flag perché serve ad avvertire che almeno una navicella del gruppo è stata colpita */
      }
      /* Controlla se il nemico è stato colpito da un missile. Caso del nemico di primo livello */
      if (livello_nemici[valore_letto.numero] == 2 && nemici_colpiti[valore_letto.numero]){
	disegna = 0;
	/* Controlla se ci sono ancora navicelle "vive" */
	for (i = n_nem_2 * valore_letto.numero, j = 0; i < n_nem_2 * valore_letto.numero + n_nem_2; i++, j = (j + 1) % n_nem_2){
	  if (nemici2l_colpiti[i] < 2)
	    disegna = 1;
	  /* Se la navicella ha ricevuto due colpi si fa sapere al nemico che è distrutta */
	  if (nemici2l_colpiti[i] == 2){
	    com_nem.nav_distrutta = j;
	    /* Si imposta a 3 il numero di colpi per evitare che si tenti di comunicare la distruzione più di una volta */
	    nemici2l_colpiti[i] = 3;
	  }
	}
        if (!disegna){
	  /* Se tutte le navicelle di secondo livello sono state distrutte comunica al nemico che è morto e lo contrassegna come tale */
	  livello_nemici[valore_letto.numero] = 0;
	  nemici_distrutti++;
	}
	/* Reimposta nuovamente il flag per evitare che si rifaccia tutto il controllo anche quando non arrivano missili*/
	nemici_colpiti[valore_letto.numero] = 0;
      }
      
      if (disegna){
	/* Calcolo della colonna iniziale e finale del nemico corrente.
	   Servono nella gestione degli scontri con l'astronave e con altri nemici*/
	int xc_iniziale = -1, xc_finale;
	if (livello_nemici[valore_letto.numero] == 1){
	  xc_iniziale = valore_letto.x;
	  xc_finale = valore_letto.x + len_nem_1;
	}else{
	  /* Nel caso del nemico di secondo livello si tiene conto delle navicelle già distrutte */
	  for (i = 0, j = 0; i < n_nem_2 * len_nem_2 + n_nem_2 - 1; i += len_nem_2 + 1, j++){		
	    if (nemici2l_colpiti[n_nem_2 * valore_letto.numero + j] < 2){
	      xc_finale = valore_letto.x + i + len_nem_2;
	      if (xc_iniziale == -1)
	        xc_iniziale = valore_letto.x + i;		  
	    }
	  }
	}


	/*Cancellazione del nemico nella sua precedente posizione*/
        if (nemici[valore_letto.numero].x >= 0)
	  /* Cancellazione nemico di primo livello */
	  if (livello_nemici[valore_letto.numero] == 1)
            for(i = 0; i < len_nem_1; i++)
	      mvaddch(nemici[valore_letto.numero].y, nemici[valore_letto.numero].x + i, ' ');
	  else
	    /* Cancellazione nemico di secondo livello 
	     * Si deve controllare che la navicella che si intende cancellare non sia già stata distrutta, in caso contrario
	     * se un altro nemico fosse vicino, lo si vedrebbe lampeggiare. In ogni caso quando le navicelle vengono distrutte, sono
	     * subito cancellate, quindi non c'è bisogno di rifarlo. */
	    for (i = 0, j = 0; i < n_nem_2 * len_nem_2 + n_nem_2 - 1; i += len_nem_2 + 1, j++)
	      if (nemici2l_colpiti[n_nem_2 * valore_letto.numero + j] < 2)
	        for (k = 0; k < len_nem_2; k++)
		  mvaddch(nemici[valore_letto.numero].y, nemici[valore_letto.numero].x + i + k, ' ');

	/* Gestione degli urti con altri nemici */
        for (i = 0; i < max_nemici; i++){
	  /* Affinché possano scontrarsi, ci si deve assicurare che non si stia confrontando il nemico corrente con se stesso,
	     che il nemico con cui lo stiamo confrontando non sia già distrutto e che i due nemici siano sulla stessa riga */
          if (i != valore_letto.numero && livello_nemici[i] != 0 && valore_letto.y == nemici[i].y){
	    /* Colonna iniziale e finale del nemico col quale confrontarlo */
	    int x2_iniziale = -1, x2_finale;
	    if (livello_nemici[i] == 1){
	      x2_iniziale = nemici[i].x;
	      x2_finale = nemici[i].x + len_nem_1;
	    }else
	      /* Nel caso del nemico di secondo livello si tiene conto delle navicelle già distrutte */
	      for (k = 0, j = 0; k < n_nem_2 * len_nem_2 + n_nem_2 - 1; k += len_nem_2 + 1, j++){		
	        if (nemici2l_colpiti[n_nem_2 * nemici[i].numero + j] < 2){
	          x2_finale = nemici[i].x + k + len_nem_2;
	          if (x2_iniziale == -1)
	            x2_iniziale = nemici[i].x + k;		  
	        }
	      }	    	    
	    /*Se urta un altro nemico, inverte la traiettoria*/
	    if (xc_iniziale - 1 == x2_finale || x2_iniziale - 1 == xc_finale)
	      com_nem.c_traiettoria = 1;
	    else
	      /* I due nemici potrebbero essere sovrapposti. Ciò succede quando uno dei due si sposta in verticale e finisce su un altro
		 o vengono creati così dall'inizio. Per evitare che comincino a rimbalzare senza riuscire a spostarsi o 
		 uno sia nascosto dietro l'altro si impone a quello corrente di fare un passo in alto*/
	      if (x2_iniziale > xc_iniziale && x2_iniziale < xc_finale){
		/* Se è già al limite superiore uno dei due attenderà che l'altro scenda! */
		if (valore_letto.y > 0){
	          com_nem.passo_su = 1;
		  valore_letto.y--;
		}
	      }
	  }
        }

	/* Se entra in collisione con l'astronave o arriva in fondo allo schermo, il giocatore ha perso*/
	if (valore_letto.y == righe_schermo)
	  game_over = 1;
	else{
	  /* Il primo requisito perché nemico e astronave siano in collisione è che siano sulla stessa riga */
	  if (astronave.y > -1 && valore_letto.y >= astronave.y + off_a_astronave){
	    /* Coordinate iniziali e finali dell'astronave */
	    int xa_iniziale = -1, xa_finale;
	    /* Calcola la colonna iniziale e finale dell'astronave nella riga considerata */
	    for (i = off_l_astronave; i < off_l_astronave + l_astronave; i++){
	      if (sch_astronave[valore_letto.y - astronave.y][i] != ' '){
		xa_finale = astronave.x + i;
		if (xa_iniziale == -1)
		  xa_iniziale = astronave.x + i;
	      }
	    }
            if ((xc_iniziale >= xa_iniziale && xc_iniziale <= xa_finale) || (xc_finale >= xa_iniziale && xc_finale <= xa_finale) || 
		(xc_iniziale <= xa_iniziale && xc_finale >= xa_finale))
	      game_over = 1;
	  }
	}

        nemici[valore_letto.numero] = valore_letto;
	/*Disegno del nemico */
	if (livello_nemici[valore_letto.numero] == 1){
          mvaddch(valore_letto.y, valore_letto.x, '(');
          mvaddch(valore_letto.y, valore_letto.x + 1, '=');
          mvaddch(valore_letto.y, valore_letto.x + 2, ')');
        }
	if (livello_nemici[valore_letto.numero] == 2){
	  for (i = 0, j = 0; i < n_nem_2; i++, j++)
	    if (nemici2l_colpiti[n_nem_2 * valore_letto.numero + i] < 2){
              mvaddch(valore_letto.y, valore_letto.x + j++, '(');
              mvaddch(valore_letto.y, valore_letto.x + j++, '^');
              mvaddch(valore_letto.y, valore_letto.x + j++, ')');
	    }else{
	      j += len_nem_2;
	    }
        }
      }
      write (pipe_nemici[valore_letto.numero], &com_nem, sizeof(com_nem)); //Si comunica al nemico se deve effettuare cambiamenti
    }
    /* FINE CONTROLLO NEMICI */

    /* CONTROLLO BOMBA */
    if (valore_letto.tipo == tipo_bomba){
      if (bombe[valore_letto.numero].x >= 0)
        mvaddch(bombe[valore_letto.numero].y, bombe[valore_letto.numero].x, ' ');
      bombe[valore_letto.numero] = valore_letto;      
      /* Disegno della bomba */
      mvaddch(bombe[valore_letto.numero].y, bombe[valore_letto.numero].x, '@');
      /* Controlla se ha colpito un'astronave */
      if (valore_letto.x >= astronave.x + off_l_astronave && valore_letto.x < astronave.x + off_l_astronave + l_astronave &&
	  valore_letto.y >= astronave.y + off_a_astronave && valore_letto.y < astronave.y + off_a_astronave + a_astronave)
	if (sch_astronave[valore_letto.y - astronave.y][valore_letto.x - astronave.x] != ' ')
	  game_over = 1;
    }
    /* FINE CONTROLLO BOMBA */

    /* Controlla se tutti i nemici sono stati distrutti. Se si il giocatore ha vinto 
     * Se il giocatore ha distrutto tutte le navicelle nemiche ma appena dopo è stato colpito da una bomba vagante, ha perso lo stesso!
     */
    if (!game_over && nemici_distrutti == max_nemici)
      game_over = 2;
      
    curs_set(0);
    refresh();
  }while(!game_over);

  /* Gioco finito. Stampa il messaggio di vittoria o sconfitta */
  char *msg;
  for (i = 0; i < colonne_schermo; i++){
    mvaddch(righe_schermo / 2, i, ' ');
    mvaddch(righe_schermo / 2 + 1, i, ' ');
  }
  if (game_over == 2)
    msg = "H A I  V I N T O ! ! !\0";
  else
    msg = "H A I  P E R S O !\0";
  mvaddstr(righe_schermo / 2, colonne_schermo / 2 - strlen(msg) / 2, msg);
  msg = "Premi INVIO per uscire\0";
  mvaddstr(righe_schermo / 2 + 1, colonne_schermo / 2 - strlen(msg) / 2, msg);
  curs_set(0);
  refresh();
}