/*
 * FILE: missile.c
 * AUTORE: Alessio Pani
 * DATA: 27/06/2009
 * PROGETTO: Space Invaders - Versione 1
 * DESCRIZIONE: Definizione della funzione di gestione del movimento di un missile lanciato dall'astronave
 */

#ifndef strutt_comunicaz_c_incluso
  #include "strutt_comunicaz.c"
#endif

/**
 * Procedura che gestisce il movimento di un missile.
 * A partire dalla posizione iniziale, si muove in diagonale verso l'alto
 * Parametri:
 *   pipeout: descrittore di scrittura della pipe per poter trasmettere al controllo i movimenti effettuati.
 *   riga: riga dello schermo da cui il missile parte
 *   colonna: colonna dello schermo da cui il missile parte
 *   colonne_schermo: Colonne dello schermo
 *   tempo: indica il tempo in microsecondi dopo il quale deve fare ogni movimento
 *   passo_x: indica di quante colonne deve spostarsi a ogni movimento. Con 0 si muove in linea retta
 *   numero: numero del missile. Serve perché il controllo possa distinguerlo dagli altri
 *   tipo: indica il codice numerico da associare all'oggetto di tipo missile.
 */
void missile(int pipeout, int riga, int colonna, int colonne_schermo, int tempo, int passo_x, int numero, int tipo){
  posizione pos; //Struttura per salvare la posizione corrente del missile e mandarla al controllo
  pos.tipo = tipo;
  pos.x = colonna + passo_x;
  pos.y = riga;
  pos.numero = numero;
  write(pipeout, &pos, sizeof(pos)); //Invio della posizione iniziale al controllo
  while (pos.y >= 0 && pos.x >= 0 && pos.x <= colonne_schermo){
    pos.y--;
    pos.x += passo_x;
    write(pipeout, &pos, sizeof(pos)); //Invio della posizione al controllo
    usleep(tempo);
  }
} 
