/*
 * FILE: nemico.c
 * AUTORE: Alessio Pani
 * DATA: 27/06/2009
 * PROGETTO: Space Invaders - Versione 1
 * DESCRIZIONE: Definizione della funzione di gestione del movimento di un nemico di primo livello
 */

#ifndef strutt_comunicaz_c_incluso
  #include "strutt_comunicaz.c"
#endif

#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include<sys/wait.h>

/**
 * Procedura che gestisce il movimento di un'astronave nemica di primo livello.
 * La sua posizione iniziale è calcolata in modo casuale in modo che si trovi in alto sullo schermo.
 * Anche la sua traiettoria è casuale. Arrivato al bordo dello schermo cambia.
 * A intervalli regolari scende di un passo verso il basso
 * Parametri:
 *  pipeout: descrittore di scrittura della pipe per poter trasmettere al controllo i movimenti effettuati.
 *  pipein: descrittore di lettura della pipe per ricevere dal controllo eventuali cambiamenti
 *  righe_schermo: righe dello schermo
 *  colonne_schermo: colonne dello schermo
 *  passo: indica di quanti caratteri deve spostarsi a ogni movimento (orizzontale e verticale) 
 *  tipo_nemico: indica il codice numerico da associare all'oggetto di tipo nemico
 *  tipo_bomba: indica il codice numerico da associare all'oggetto di tipo bomba
 *  numero: numero da associare al nemico affinché il controllo possa distinguerlo dagli altri
 */

void nemico(int pipeout, int pipein, int righe_schermo, int colonne_schermo, int passo, int tipo_nemico, int tipo_bomba, int numero){
  srand(time(NULL) + numero);

  posizione pos;
  cont_nem contr;

  int traiettoria, dx, muori = 0;
  int i = 0;
  int pid_bomba = -1;

  /*
   * L'intervallo di tempo per il lancio della bomba viene scelto in modo casuale fra 40 e 110.
   * Non si assegna un valore a priori per evitare piogge di bombe a intervalli regolari
   */
  int int_bomba = 40 + (rand()%150), cnt_bomba = int_bomba;
  
  /* Posizione iniziale ed inizializzazione degli altri dati da inviare al controllo*/
  pos.x = 1 + (rand()%colonne_schermo);
  pos.y = 1 + (rand()%(righe_schermo / 4));
  pos.tipo = tipo_nemico;
  pos.numero = numero;

  /* Invio della posizione iniziale al controllo */
  write(pipeout, &pos, sizeof(pos));

  traiettoria = rand()%2;
  while(!muori){

    /* Riceve il messaggio da parte del controllo */   
    read(pipein, &contr, sizeof(contr));

    /* Se il controllo impone il cambio di traiettoria, la inverte e si sposta di due passi anziché uno */
    if (contr.c_traiettoria){
      traiettoria = (traiettoria + 1) % 2;
      dx = 2 * passo;
    }else
      dx = passo;

    /* Il controllo può aver comunicato che il nemico è stato ucciso */
    muori = contr.c_livello;

    /* Il controllo può aver imposto di fare un passo in alto */
    pos.y -= contr.passo_su;

    /* Generazione delle nuove coordinate */
    if (traiettoria){
      pos.x += dx;
      if (pos.x >= colonne_schermo - 1){
        traiettoria = 0;
	pos.x = colonne_schermo - 1;
      }
    }else{
      pos.x -= dx;
      if (pos.x <= 0){
	traiettoria = 1;
        pos.x = 0;
      }
    }

    /* A intervalli regolari scende di un passo verso il basso */
    i = (i + 1)%60;
    if (!i) pos.y += passo;

    if (!muori){
      /* Può lanciare una bomba alla volta. Se una è stata già lanciata, si comincia a contare il tempo quando scompare 
       * Per verificare se è scomparsa si verifica se il processo associato è terminato
       */
      if (pid_bomba == -1 || waitpid(pid_bomba, NULL, WNOHANG)){
        cnt_bomba = --cnt_bomba % int_bomba;
        if (!cnt_bomba) /* Sgancia una bomba ad intervalli regolari */
	  switch(pid_bomba = fork()){
	    case 0:
              bomba(pipeout, pos.y + 1, pos.x, righe_schermo, 90000, numero, tipo_bomba);
	      exit(0);
	  }
      }
      /* Invio delle coordinate al controllo */
      write(pipeout, &pos, sizeof(pos));     
    }
    else /* Il nemico di primo livello è stato distrutto. Crea quello di secondo livello in un nuovo processo. */
      switch(fork()){
        case 0:
	 nemico2l(pipeout, pipein, pos.y, pos.x, righe_schermo, colonne_schermo, passo, tipo_nemico, tipo_bomba, numero);
	 exit(0);
      }
    usleep(100000);
  }
} 