/*
 * FILE: nemico2l.c
 * AUTORE: Alessio Pani
 * DATA: 27/06/2009
 * PROGETTO: Space Invaders - Versione 1
 * DESCRIZIONE: Definizione della funzione di gestione del movimento di un nemico di secondo livello
 */

#ifndef strutt_comunicaz_c_incluso
  #include "strutt_comunicaz.c"
#endif

#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include<sys/wait.h>

/**
 * Procedura che gestisce il movimento di un'astronave nemica di secondo livello.
 * La sua traiettoria è casuale. Arrivato al bordo dello schermo cambia.
 * A intervalli regolari scende di un passo verso il basso
 * Parametri:
 *  pipeout: descrittore di scrittura della pipe per poter trasmettere al controllo i movimenti effettuati.
 *  pipein: descrittore di lettura della pipe per ricevere dal controllo eventuali cambiamenti
 *  riga: riga dalla quale deve partire
 *  colonna: colonna dalla quale deve partire
 *  righe_schermo: righe dello schermo
 *  colonne_schermo: colonne dello schermo
 *  passo: indica di quanti caratteri deve spostarsi a ogni movimento (orizzontale e verticale) 
 *  tipo_nemico: indica il codice numerico da associare all'oggetto di tipo nemico
 *  tipo_bomba: indica il codice numerico da associare all'oggetto di tipo bomba
 *  int numero: numero da associare al nemico affinché il controllo possa distinguerlo dagli altri
 */
void nemico2l(int pipeout, int pipein, int riga, int colonna, int righe_schermo, int colonne_schermo, int passo, int tipo_nemico, int tipo_bomba, int numero){
  posizione pos;
  cont_nem contr;
  int traiettoria, dx, muori = 0;
  int i = 0, j;

  int len_nem = 3; /* Numero di caratteri occupati dalla singola navicella */
  int n_navicelle = 4;
  int navicella_distrutta[4] = {0}, navicelle_distrutte = 0;

  int pid_bomba = -1;

  /*
   * Visto che le bombe devono condividere lo stesso numero, ogni navicella ne lancia una a turno.
   * L'intervallo di tempo per il lancio della bomba viene scelto in modo casuale fra 40 e 110.
   */
  int int_bomba = 40 + (rand()%150), cnt_bomba = int_bomba, turno_bomba = -1;

  srand(time(NULL) + numero);

  /* Posizione iniziale ed inizializzazione degli altri dati da inviare al controllo*/
  pos.x = colonna;
  pos.y = riga;
  pos.tipo = tipo_nemico;
  pos.numero = numero;

  /* Invio della posizione iniziale al controllo */
  write(pipeout, &pos, sizeof(pos));

  traiettoria = rand()%2;
  while(!muori){

    /* Riceve il messaggio da parte del controllo */   
    read(pipein, &contr, sizeof(contr));

    /* Se il controllo impone il cambio di traiettoria, la inverte e si sposta di due passi anziché uno */  
    if (contr.c_traiettoria){
      traiettoria = (traiettoria + 1) % 2;
      dx = 2 * passo;
    }else
      dx = passo;

    /* Il controllo può aver comunicato che una delle navicelle è stata colpita */
    if (contr.nav_distrutta != -1){
      navicella_distrutta[contr.nav_distrutta] = 1;
      navicelle_distrutte++;
      /* Se tutte le navicelle sono state distrutte il nemico di secondo livello si considera morto */
      if (navicelle_distrutte == n_navicelle)
        muori = 1;
    }

    /* Il controllo può aver imposto di fare un passo in alto */
    pos.y -= contr.passo_su;

    /* Generazione delle nuove coordinate */
    if (traiettoria){
      pos.x += dx;
      if (pos.x >= colonne_schermo - 16){
        traiettoria = 0;
	pos.x = colonne_schermo - 16;
      }
    }else{
      pos.x -= dx;
      if (pos.x <= 0){
	traiettoria = 1;
	pos.x = 0;
      }
    }

    /* A intervalli regolari scende di un passo verso il basso */
    i = (i + 1)%60;
    if (!i) pos.y++;
    if (!muori){
      /* Può lanciare una bomba alla volta. Se una è stata già lanciata, si comincia a contare il tempo quando scompare 
       * Per verificare se è scomparsa si verifica se il processo associato è terminato
       */
      if (pid_bomba == -1 || waitpid(pid_bomba, NULL, WNOHANG)){
        cnt_bomba = --cnt_bomba % int_bomba;
        if (!cnt_bomba){ /* Sgancia una bomba ad intervalli regolari */
	  do{
	    turno_bomba = (turno_bomba + 1) % n_navicelle;
	  }while(navicella_distrutta[turno_bomba]);
	  switch(pid_bomba = fork()){
	    case 0:
              bomba(pipeout, pos.y + 1, pos.x + (turno_bomba) * (n_navicelle + 1), righe_schermo, 90000, numero, tipo_bomba);
	      exit(0);
	  }
	  /* Determina quale navicella lancerà la bomba la prossima volta, scartando quelle distrutte */
	}
      }
      /* Invio delle coordinate al controllo */
      write(pipeout, &pos, sizeof(pos));     
    }
    usleep(100000);
  }
}  
