/*
 * FILE: spaceinv.c
 * AUTORE: Alessio Pani
 * DATA: 27/06/2009
 * PROGETTO: Space Invaders - Versione 1
 * DESCRIZIONE: Avvio del gioco. Le costanti permettono di impostare i parametri.
 *	Viene creato un processo per ogni oggetto inizialmente mostrato sullo schermo e per il controllo
 */

#include <stdio.h>
#include <curses.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

/* Codice dei tasti da utilizzare per il gioco*/
#define SINISTRA 68
#define DESTRA 67
#define SPARA 32

/* Dimensione dello schermo */
#define MAXX 80
#define MAXY 24

/* Lunghezza in caratteri degli spostamenti*/
#define PASSO_ASTRONAVE 1
#define PASSO_NEMICI 1

//Numero di nemici da creare
#define NEMICI 10

/*Numero massimo di missili che si possono lanciare prima che i precedenti siano scomparsi dallo schermo
  Deve essere multiplo di 2 perché ne vengono sparati 2 per volta*/
#define MAX_MISSILI 10

/* Codice da associare ai vari oggetti da mostrare. Devono essere tutti diversi perché
 * servono a indicare al controllo da chi arrivano i messaggi
 */
#define TIPO_ASTRONAVE 0
#define TIPO_NEMICO 1
#define TIPO_MISSILE 2
#define TIPO_BOMBA 3

int main(){
  int filedes[2], pipe_nemici[2 * NEMICI], pipe_scrittura_nemici[NEMICI];
  int pid_astronave, pid_nemici[NEMICI], pid_controllo, pid_uscita, i;
  int nemici_counter = -1;
  initscr();
  noecho();  
  curs_set(0);

  for (i = 0; i < NEMICI; i++)
    pid_nemici[i] = -2;

  /* Creazione della pipe per la comunicazione fra i processi */
  if (pipe(filedes) == -1){
    perror("Errore nella creazione della pipe");
    exit(1);
  }

  /* Creazione delle pipe per la comunicazione fra controllo e nemici */
  for (i = 0; i < (NEMICI * 2); i += 2)
    if (pipe(pipe_nemici + i) == -1){ 
      perror("Errore nella creazione della pipe");
      exit(1);
    }
  

  switch(pid_astronave = fork()){
    case -1:
      perror("Errore nell'esecuzione della fork");
      exit(1);
    case 0:  /* Ci troviamo nel processo figlio */
      close(filedes[0]); //Chiude il descrittore di lettura della pipe
      astronave(filedes[1], MAXY, MAXX, DESTRA, SINISTRA, SPARA, PASSO_ASTRONAVE, MAX_MISSILI, TIPO_ASTRONAVE, TIPO_MISSILE);
      break;
    default: /* Ci troviamo nel processo padre */
      switch (pid_controllo = fork()){
        case -1:
          perror("Errore nell'esecuzione della fork");
          exit(1);
        case 0:
	  close(filedes[1]); /* Chiude il descrittore di scrittura della pipe */
	  for (i = 0; i < 2 * NEMICI; i += 2){
	    close(pipe_nemici[i]); /* Chiude il descrittore di lettura della pipe */
	    pipe_scrittura_nemici[i/2] = pipe_nemici[i + 1]; /* Copia i descrittori di scrittura in un apposito array */
	  }
	  controllo(filedes[0], pipe_scrittura_nemici, MAXY, MAXX, NEMICI, MAX_MISSILI, TIPO_ASTRONAVE, TIPO_MISSILE, TIPO_NEMICO, TIPO_BOMBA);
	  break;
        default:     
	  close(filedes[0]);
	  /* Nel processo padre vengono creati tanti processi quanti sono i nemici da far comparire */
	  do{	      
	    nemici_counter++;
	    pid_nemici[nemici_counter] = fork();
	    if (pid_nemici[nemici_counter] == -1){
              perror("Errore nell'esecuzione della fork");
              exit(1);
	    }
	  }while(pid_nemici[nemici_counter] != 0 && (nemici_counter < NEMICI - 1));
	  /* Il ciclo di seguito determina se ci si trova nel processo padre o in uno dei figli */
	  int padre = 1;
	  for (i = 0; i < NEMICI; i++){
   	    if (pid_nemici[i] == 0){
	      padre = 0;
	      break;
	    }
	  }
	  /* Se ci troviamo in uno dei figli crea un nemico */
	  if (!padre){
	    nemico(filedes[1], pipe_nemici[2 * nemici_counter], MAXY, MAXX, PASSO_NEMICI, TIPO_NEMICO, TIPO_BOMBA, nemici_counter);
	  /* Altrimenti si svolgono le restanti operazioni del processo padre */
	  }else{
	    /* Se il processo di controllo termina, significa che il gioco è finito */
            waitpid(pid_controllo, NULL, 0);	    
	    for (i = 0; i < NEMICI; i++)
	      kill(pid_nemici[i], 1);
	    kill(pid_astronave, 1);  
	    while ((int)getchar() != (int)'\r'); /* Premere INVIO per uscire! */
	    endwin();	    
	  }
      }
	
  }
  return 0;
}